/*

File: Pause.c
Purpose: Usage of Timer2 on PIC18F4550 to produce delays.
Compiler: MPLAB C18 v.3.22 Student Edition in MPLAB IDE v.8.15a.

Author: Predrag Pei (eng. Peshich), Belgrade, Serbia.
Board: EasyPIC3 by Mikroelektronika.
January 2009.

This is the main file for delay function Pause() for delay in seconds and fragments of a second.

How the program is working: Eanbling the Timer2 and assuming that EasyPIC3 is clocked by 8MHz primary crystall osc. the source for Timer2 is Fosc/4 which means that the incoming clock stream is divided by 4. Because we are using HSPLL the controller and periferals are clocked by 48Mhz, the timer TMR2 is clocked by 12MHz clock stream. This stream enters in prescaler which divides it by 16. This produces 750kHz clock stream which enters in comparator.
The PR2 register is set to 0xFA (250 dec.) which means that comparator will produce one pulse on every 250 pulses from timer TMR2. The result is that that we have 3kHz outcoming stream of pulses from comparator to postscaler which is configured to 1:3 to produce the stream to TMR2IF (interrupt flag). Now, we have 1kHz which triggers the inerrupt.
By this way we have near 1mS pulse on TMR2IF!

Program first jump to the function Pause() where we initialise the necessery variables and start the hardware Timer2 to produce interrupts from TMR2IF.
On every occurence of interrupt from TMR2IF the program jumps to ISR Timer2() where we increment the variable Sec in fragments of 0.001 for counting the seconds.
During that time the program monitors the value of variable Sec and when it reaches the desired value, stops the hardware Timer2 end exit function Pause().

*/


#include "p18f4550.h"
#include "Config_Bits.h"
#include "Pause.h"


       
volatile float Sec=0.0;		          // Global variable. It`s value can be changed from anywhere in the program.

#pragma interrupt Timer2						// Tell the compiler that ISR for high-interrupt is Timer2.
void Timer2(void)										// The body of high-interrupt ISR starts here. 
{
	PIR1bits.TMR2IF=0;								// Immediately clear the interrupt flag.			
		Sec+=0.001;                     // Increase Sec in miliseconds. Remember, this means that one mS is passed!
}

#pragma code Address_Of_HISR_Is_At=0x08		// We shall use high-interrupt. The address of high-interrupt vector (HISR) is at address 0x08.
void On_Timer2_Interrupt (void)						// On every occurence of interrupt from Timer2...
{
	_asm
	goto	Timer2													  // ... jump to high-interrupt ISR Timer2!
	_endasm
}
#pragma code                              // Return from #pragma to code section.

void Pause(float Seconds)
{
  Sec=0.0;                  // Initialise necessery counters to zero.
	IPR1bits.TMR2IP=1;				// Interrupt priority bit set to 1 - high priority.
	PIE1bits.TMR2IE=1;				// Interrupt on TMR2IF enabled.
	 
	T2CON=0b00010110;					// Timer2 config register:	prescaler (bits 0-1):				divide by 16
	 													//													timer2 ON/OFF (bit 2):			TMR2ON=1 - ON
	 													// 													postscaler (bits 3-7):			1:3
	PR2=0xFA;									// Period register set to 250 (dec.).
	INTCONbits.PEIE=1;				// Enable peripheral interrupt sources.
	INTCONbits.GIE=1;					// Enable global interrupts.
	while(Sec<=Seconds);		  // Wait for the given seconds...
	T2CONbits.TMR2ON=0;				// ... than stop theTimer2, TMR2ON=0 - OFF...
		INTCONbits.PEIE=0;			// ... disable peripheral interrupt sources...
		INTCONbits.GIE=0;				// ... and disble global interrupts.	
}


void main()
{
 TRISD=0x00; LATD=0x00;    // Prepare port D to be the output and that LED`s connected to it are turned OFF.
 
 while(1)
 {
   Pause(0.581);   // Wait the desired amount of time in seconds. Change the value and see the result.
// LATD=!LATD;        This way only LED connected to bit 0 is flashing, not the whole port (what is my intention).   
   LATDbits.LATD0=!LATDbits.LATD0;   // So, invert port D, bit by bit!
   LATDbits.LATD1=!LATDbits.LATD1;
   LATDbits.LATD2=!LATDbits.LATD2;
   LATDbits.LATD3=!LATDbits.LATD3;
   LATDbits.LATD4=!LATDbits.LATD4;
   LATDbits.LATD5=!LATDbits.LATD5;
   LATDbits.LATD6=!LATDbits.LATD6;
   LATDbits.LATD7=!LATDbits.LATD7;
 }
}   
